/*
	*
	* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

// Local Headers
#include "Global.hpp"
#include "DesQShell.hpp"

// LibDesQ Headers
#include <desq/DesQUtils.hpp>
#include <desqui/DesQApplication.hpp>

// Wayland Headers
#include <desqwl/DesQWlGlobal.hpp>
#include <desqwl/Registry.hpp>
#include <desqwl/LayerShell.hpp>

int main( int argc, char **argv ) {

	qInstallMessageHandler( Logger );
	qputenv( "QT_WAYLAND_USE_BYPASSWINDOWMANAGERHINT", QByteArrayLiteral( "1" ) );

	QCoreApplication::setAttribute( Qt::AA_EnableHighDpiScaling );

	DesQApplication app( "DesQShell", argc, argv );

	if ( app.isRunning() ) {
		if ( argv[ 1 ] == QString( "--dropdown" ) ) {
			app.sendMessage( "dropdown" );
			return 0;
		}

		if ( argv[ 1 ] == QString( "--background" ) ) {
			if ( argc == 3 )
				app.sendMessage( QString( "background\n%1" ).arg( argv[ 2 ] ) );

			return 0;
		}

		else if ( argv[ 1 ] == QString( "--lockscreen" ) ) {
			app.sendMessage( "lockscreen" );
			return 0;
		}
	}

    QCommandLineParser parser;
    parser.addHelpOption();         // Help
    parser.addVersionOption();      // Version

	/* Optional: Path where the search begins */
    parser.addOption( { "background", "The path to the background file", "image file" } );

	/* Process the CLI args */
    parser.process( app );

	DesQShell *shell;

	if ( parser.isSet( "background" ) )
		shell = new DesQShell( parser.value( "background" ) );

	else
		shell = new DesQShell();

	QObject::connect( &app, &DesQApplication::messageReceived, shell, &DesQShell::handleMessages );

	shell->show();
	qDebug() << "Starting desq-shell";

	if ( DesQWayland::isRunning() ) {

		wl_display *display = DesQWayland::getWlDisplay();
		if ( not display ) {
			qDebug() << "Unable to acquire wl_display from the compositor.";
			qDebug() << "Your experience will be severly limited.";

			return 0;
		}

		WaylandRegistry *registry = new WaylandRegistry( display );
		QObject::connect(
			registry, &WaylandRegistry::errorOccured, [=]( WaylandRegistry::ErrorType et ) {
				qDebug() << "Error caused on registry" << et;
				qDebug() << "Valiantly trying to continue...";
				// exit(0);
			}
		);
		registry->initialize();

		LayerSurface *cls = new LayerSurface( shell->windowHandle() );
		cls->setup( registry->lShell(), LayerShell::Background );
		cls->setAnchors(LayerSurface::Top | LayerSurface::Bottom | LayerSurface::Left | LayerSurface::Right);
		cls->setExclusiveZone( -1 );
		cls->setKeyboardInteractivity( true );
		cls->apply();

		shell->setupWayland( registry );
	}

	return app.exec();
};
