/*
    *
    * This file is taken from libcsys <https://gitlab.com/cubocore/corestuff>
    * Further modifications may br done as per the requirements of DesQ
    * All bug repots are to be filed with DesQ and not libcsys
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QLabel>
#include <QScreen>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QApplication>

#include "Global.hpp"

#include "LogoutDialog.hpp"

LogoutDialog::LogoutDialog() : QDialog() {

	setWindowFlags( Qt::Dialog | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint );
	setWindowModality( Qt::ApplicationModal );

	powerDBus = new Power( this );

	logOutBtn = new QCommandLinkButton( "Logout", "Logout as the current user" );
	logOutBtn->setIcon( QIcon::fromTheme( "system-log-out" ) );
    logOutBtn->setIconSize( QSize( 36, 36 ) );
	connect( logOutBtn, &QCommandLinkButton::clicked, [=]() {
		QDialog::close();			// We do not want this dialog to appear while saving the session

		QProcess proc;
		proc.start( "desq-session", QStringList() << "--logout" );
		proc.waitForFinished();

		/* We manually close this app: May not be necessary */
		qApp->quit();
	} );

	suspendBtn = new QCommandLinkButton( "Suspend", "Suspend the system to RAM" );
	suspendBtn->setIcon( QIcon::fromTheme( "system-suspend" ) );
    suspendBtn->setIconSize( QSize( 36, 36 ) );
	connect( suspendBtn, &QCommandLinkButton::clicked, [=]() {
		QDialog::close();			// We do not want this dialog to appear while saving the session

		QProcess proc;
		proc.start( "desq-session", QStringList() << "--suspend" );
		proc.waitForFinished();
	} );

	if ( not powerDBus->systemCanSuspend() )
        suspendBtn->setDisabled( true );

	hibernateBtn = new QCommandLinkButton( "Hibernate", "Suspend the system to Disk" );
	hibernateBtn->setIcon( QIcon::fromTheme( "system-suspend-hibernate" ) );
    hibernateBtn->setIconSize( QSize( 36, 36 ) );
	connect( hibernateBtn, &QCommandLinkButton::clicked, [=]() {
		QDialog::close();			// We do not want this dialog to appear while saving the session

		QProcess proc;
		proc.start( "desq-session", QStringList() << "--hibernate" );
		proc.waitForFinished();
	} );

	if ( not powerDBus->systemCanHibernate() )
        hibernateBtn->setDisabled( true );

    powerOffBtn = new QCommandLinkButton( "Power Off", "Power off the system" );
	powerOffBtn->setIcon( QIcon::fromTheme( "system-shutdown" ) );
    powerOffBtn->setIconSize( QSize( 36, 36 ) );
	connect( powerOffBtn, &QCommandLinkButton::clicked, [=]() {
		QDialog::close();			// We do not want this dialog to appear while saving the session

		QProcess proc;
		proc.start( "desq-session", QStringList() << "--shutdown" );
		proc.waitForFinished();

		/* We manually close this app: May not be necessary */
		qApp->quit();
	} );

	if ( not powerDBus->systemCanHalt() )
        powerOffBtn->setDisabled( true );

    rebootBtn = new QCommandLinkButton( "Reboot", "Reboot the system" );
	rebootBtn->setIcon( QIcon::fromTheme( "system-reboot" ) );
    rebootBtn->setIconSize( QSize( 36, 36 ) );
	connect( rebootBtn, &QCommandLinkButton::clicked, [=]() {
		QDialog::close();			// We do not want this dialog to appear while saving the session

		QProcess proc;
		proc.start( "desq-session", QStringList() << "--reboot" );
		proc.waitForFinished();

		/* We manually close this app: May not be necessary */
		qApp->quit();
	} );

	if ( not powerDBus->systemCanReboot() )
        rebootBtn->setDisabled( true );

	cancelBtn = new QToolButton();
	cancelBtn->setIcon( QIcon::fromTheme( "dialog-close" ) );
    cancelBtn->setIconSize( QSize( 24, 24 ) );
	connect( cancelBtn, &QToolButton::clicked, this, &QDialog::reject );

	QHBoxLayout *titleLyt = new QHBoxLayout();
	titleLyt->addWidget( new QLabel( "Who do you want to do?" ) );
	titleLyt->addStretch();
	titleLyt->addWidget( cancelBtn );

	QVBoxLayout *lyt = new QVBoxLayout();
	lyt->addLayout( titleLyt );
	lyt->addWidget( logOutBtn );
	lyt->addWidget( suspendBtn );
	lyt->addWidget( hibernateBtn );
	lyt->addWidget( powerOffBtn );
	lyt->addWidget( rebootBtn );

	setLayout( lyt );

	setFixedSize( 250, 384 );

	QSize scrnSize = QApplication::primaryScreen()->size();
	int h = ( scrnSize.width() - 250 ) / 2;
	int v = ( scrnSize.height() - 384 ) / 2;

	move( h, v );
};
