/*
	*
	* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project ( https://gitlab.com/desq/ )
	* Qt5 example of analog clock, and Lenovo K8 Note's clock widget
	* are the inspiration for this. Some parts of the code are taken
	* from Qt5's example
	*
	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*
	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*
	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QtWidgets>
#include <desq/SysInfo.hpp>

#include "ResourcesWidget.hpp"

ResourcesWidget::ResourcesWidget( QWidget *parent ) : QWidget( parent ) {

	mSysInfo = SysInfo::sysInfoObject();
	mTimer = new QBasicTimer();

	numRes = 2;
	labels = QStringList( { "CPU", "RAM" } );

	QGridLayout *lyt = new QGridLayout();
	for( int i = 0; i < numRes; i++ ) {
		QProgressBar *pb = new QProgressBar( this );
		pb->setOrientation( Qt::Vertical );
		pb->setStyleSheet(
			"QProgressBar{"
			"	border: 0.5px solid palette(Window);"
			"	background-color: transparent;"
			"	border-radius: none;"
			"	width: 20px;"
			"}"
			"QProgressBar::chunk {"
			"	background-color: palette(Highlight);"
			"	margin: 0.5px;"
			"}"
		);
		pb->setRange( 0, 100 );
		pb->setFormat( "" );
		resWidgets << pb;

		QLabel *lbl = new QLabel( labels[ i ] );
		lbl->setAlignment( Qt::AlignCenter );
		resLabels << lbl;

		lyt->addWidget( pb, 0, i, Qt::AlignCenter );
		lyt->addWidget( lbl, 1, i, Qt::AlignCenter );
	}

	setLayout( lyt );

	opacity = new QGraphicsOpacityEffect( this );
	opacity->setOpacity( 0.4 );
	setGraphicsEffect( opacity );

    mTimer->start( 1000, this );

    setMouseTracking( true );
};

void ResourcesWidget::timerEvent( QTimerEvent *tEvent ) {

    if ( tEvent->timerId() == mTimer->timerId() ) {
		QList<double> percents;
		percents << mSysInfo->getCpuUsage().at( 0 );

		QList<double> ram = mSysInfo->getRamUsage();
		percents << 100.0 * ram[ 0 ] / ram[ 1 ];

		for( int i = 0; i < numRes; i++ ) {
			resWidgets[ i ]->setValue( qRound( percents.at( i ) ) );
			resLabels[ i ]->setText( QString( "%1: %2 %" ).arg( labels.at( i ) ).arg( percents[ i ], 0, 'f', 2 ) );
		}
	}

	QWidget::timerEvent( tEvent );
};

void ResourcesWidget::enterEvent( QEvent *event ) {

	opacity->setOpacity( 1.0 );
	event->accept();
};

void ResourcesWidget::leaveEvent( QEvent *event ) {

	opacity->setOpacity( 0.4 );
	event->accept();
};
